﻿using System.Collections.Generic;
using UnityEngine;
using System;
using System.Net.Sockets;
using System.Threading;
using UnityEngine.UI;
using UnityEngine.EventSystems;

public class NetworkControllerScript : MonoBehaviour
{
    private TcpClient TcpClient;
    private NetworkStream NetworkStream;

    public GameObject Panel;
    public Button ConnectButton;
    public Text MyConsole;
    public InputField IPInputField;
    public InputField PortInputField;
    public bool Networking;
    private bool ConnectionSet;
    private GraphicRaycaster raycaster;
    private EventSystem eventSystem;

    void Start()
    {
        MyConsole.text = "IP: 192.168.43.102 \n" + "Port: 90 \n";
        ConnectionSet = false;

        EventTrigger eT = Panel.AddComponent<EventTrigger>();
        PointerEventData pointerEventData = new PointerEventData(eventSystem);
        eT.OnPointerDown(pointerEventData);

        raycaster = Panel.GetComponent<GraphicRaycaster>();
        eventSystem = GetComponent<EventSystem>();

        IPInputField.text = "192.168.43.102";
        PortInputField.text = "90";

        if (Networking)
        {
            Panel.SetActive(true);
            ConnectButton.onClick.AddListener(delegate { OnConnectButtonClick(ConnectButton); });
        }
        else
        {
            Panel.SetActive(false);
        }
    }
    
    void Update()
    {
        
    }

    public void OnConnectButtonClick( Button button)
    {
        if(Networking)
        {
            MyConsole.text += $"Connecting to {IPInputField.text}:{PortInputField.text}... \n";
            if (Int32.TryParse(PortInputField.text, out int result))
                if (NetworkConnector.TCPStartConnection(IPInputField.text, result, out TcpClient tcpClient, out NetworkStream networkStream, ref MyConsole))
                {
                    TcpClient = tcpClient;
                    NetworkStream = networkStream;
                    MyConsole.text = "Connected!";
                    Thread.Sleep(500);
                    Panel.SetActive(false);
                    ConnectionSet = true;
                }   
        }
    }

    public void OnPointerDown()
    {
        PointerEventData pointerEventData = new PointerEventData(eventSystem);
        pointerEventData.position = Input.mousePosition;
        List<RaycastResult> raycastResults = new List<RaycastResult>();
        raycaster.Raycast(pointerEventData, raycastResults);
    }

    public void SendFeedback(List<FeedbackObject> feedbackObjects)
    {
        if(Networking && ConnectionSet)
        {
            foreach(FeedbackObject fb in feedbackObjects)
            {
                NetworkConnector.TCPSend(NetworkStream, fb.ID, fb.Action);
            }
        }
    }

    public List<InstructionObject> GetInstructionObjects()
    {
        List<InstructionObject> output = new List<InstructionObject>();
        if(Networking && ConnectionSet)
        {
            Dictionary<string, int> RecieveDictionary = new Dictionary<string, int>();
            RecieveDictionary.Add("u", 0);
            RecieveDictionary.Add("r", 1);
            RecieveDictionary.Add("d", 2);
            RecieveDictionary.Add("l", 3);
            RecieveDictionary.Add("U", 4);
            RecieveDictionary.Add("R", 5);
            RecieveDictionary.Add("D", 6);
            RecieveDictionary.Add("L", 7);
            RecieveDictionary.Add("k", 8);



            while (NetworkStream.DataAvailable)
            {
                if (NetworkConnector.TCPReceive(NetworkStream, out string UID, out string Name, out string Action))
                {
                    if(RecieveDictionary.TryGetValue(Action, out int ActionValue))
                    {
                        output.Add(
                        new InstructionObject()
                        {
                            ID = UID,
                            Name = Name,
                            Action = ActionValue
                        }
                        );
                    }

                }
            }
        }
        else
        {

        }
        return output;
    }

}

public class FeedbackObject
{
    public string ID { get; set; }
    public string Action { get; set; }
}

public class InstructionObject
{
    public string ID { get; set; }
    public string Name { get; set; }
    public int Action { get; set; }
}

public static class NetworkConnector
{
    public static bool TCPStartConnection(String server, Int32 port, out TcpClient client, out NetworkStream stream, ref Text MyConsole)
    {
        bool connected = false;
        TcpClient Client = null;
        int N = 3;
        int n = 1;
        do
        {
            try
            {
                Console.Clear();
                MyConsole.text = "Connecting...";
                Client = new TcpClient(server, port);
                connected = true;
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
                if (n++ == N)
                    break;
                MyConsole.text += $"\n Attempt: {n}/{N}";
                Thread.Sleep(1000);
            }
        } while (!connected);

        if(connected)
            MyConsole.text = "Connected!";

        client = Client;
        stream = client.GetStream();

        return connected;
    }
    public static bool TCPSend(NetworkStream stream, string UID, string Action)
    {
        bool succeed = false;
        Byte[] data = new byte[64];

        byte[] uid = System.Text.Encoding.ASCII.GetBytes(UID);
        int i = 0;
        for (; i < uid.Length; i++)
            data[i] = uid[i];
        for (; i < 32; i++)
            data[i] = 32;

        byte[] action = System.Text.Encoding.ASCII.GetBytes(Action);
        for (i = 0; i < action.Length; i++)
            data[i + 32] = action[i];
        for (; i < 32; i++)
            data[i + 32] = 32;

        try
        {
            stream.Write(data, 0, 64);
            succeed = true;
        }
        catch (ArgumentNullException e)
        {
            Console.WriteLine("ArgumentNullException: {0}", e);
        }
        catch (SocketException e)
        {
            Console.WriteLine("SocketException: {0}", e);
        }
        catch (Exception e)
        {
            Console.WriteLine("Exception: {0}", e);
        }

        return succeed;
    }

    public static bool TCPReceive(NetworkStream stream, out string UID, out string Name, out string Action)
    {
        bool succeed = false;
        UID = Name = Action = "";

        Byte[] data = new Byte[3 * 32];

        try
        {
            stream.Read(data, 0, 3 * 32);
            succeed = true;
        }
        catch (ArgumentNullException e)
        {
            Console.WriteLine("ArgumentNullException: {0}", e);
        }
        catch (SocketException e)
        {
            Console.WriteLine("SocketException: {0}", e);
        }
        catch (Exception e)
        {
            Console.WriteLine("Exception: {0}", e);
        }

        byte[] uid = new byte[32];
        for (int i = 0; i < 32; i++)
            uid[i] = data[i];

        byte[] name = new byte[32];
        for (int i = 0; i < 32; i++)
            name[i] = data[i + 32];

        byte[] action = new byte[32];
        for (int i = 0; i < 32; i++)
            action[i] = data[i + 64];



        UID = System.Text.Encoding.ASCII.GetString(uid).Trim();
        Name = System.Text.Encoding.ASCII.GetString(name).Trim();
        Action = System.Text.Encoding.ASCII.GetString(action).Trim();

        return succeed;
    }

    public static bool TCPCloseConnection(NetworkStream stream, TcpClient client)
    {
        bool succeed = false;

        try
        {
            stream.Close();
            client.Close();
            succeed = true;
        }
        catch (ArgumentNullException e)
        {
            Console.WriteLine("ArgumentNullException: {0}", e);
        }
        catch (SocketException e)
        {
            Console.WriteLine("SocketException: {0}", e);
        }
        catch (Exception e)
        {
            Console.WriteLine("Exception: {0}", e);
        }

        return succeed;
    }
}