﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Animations;

public class PlayerController : MonoBehaviour
{
    public float heightOffset;
    public int[] currentGridPos = new int[2] { 20, 20 };
    private int[] requestedGridPos = new int[2] { 20, 20 };

    public NetworkControllerScript networkControllerScript;
    private Vector2 currentPos;
    public Vector2 requestedPos;
    public GridLayout gridLayout;
    private bool currentlyMoving;
    public GameManager manager;
    public PlayerScore score;
    public float timeOfJump;
    private float jumpTimer;
    [Space(10)]
    [Header("haha")]
    public string ID;

    private List<IEffect> effects;
    [SerializeField]
    private Color color = Color.red;
    public MeshRenderer rend;
    public float cooldown;
    private float cooldownTimer;
    private bool onCooldown;
    private Rigidbody rigi;
    public AnimationCurve curve;
    // Start is called before the first frame update
    public void SetColor(Color colorek)
    {
        color = colorek;
        rend.material.color = colorek;
    }



    void Start()
    {

        //animator = GetComponentInChildren<Animator>();
        //initialize to random grid position
        //animator.SetFloat("jumpTime", timeOfJump);
        // rigi = GetComponent<Rigidbody>();
        score = GetComponent<PlayerScore>();
        currentGridPos = PosToGridPos(new Vector2(transform.position.x, transform.position.z));
        requestedGridPos = new int[] { currentGridPos[0], currentGridPos[1] };
        currentPos = GridPosToPos(currentGridPos);
        transform.position = new Vector3(currentPos.x, transform.position.y, currentPos.y);
        jumpTimer = 0;
        currentlyMoving = false;
    }

    // Update is called once per frame
    void Update()
    {
        if (currentlyMoving)
        {
            jumpTimer += Time.deltaTime;

            // Vector2 dir = (requestedPos - currentPos);
            // dir.Normalize();

            Vector2 dir = Vector2.Lerp(currentPos, requestedPos, jumpTimer / timeOfJump);

            transform.position = new Vector3(dir.x, (curve.Evaluate(jumpTimer / timeOfJump) + heightOffset), dir.y);
            if (jumpTimer > timeOfJump)
            {
                jumpTimer = 0;
                OnFinishedMoving();
            }

        }
        else
        {
            if (onCooldown)
            {
                cooldownTimer += Time.deltaTime;
                if (cooldownTimer > cooldown)
                {
                    cooldownTimer = 0;
                    onCooldown = false;
                }
                return;
            }


            if (requestedGridPos[0] != currentGridPos[0] || requestedGridPos[1] != currentGridPos[1])
            {
                OnStartMoving();



            }
        }
    }

    void OnStartMoving()
    {
        // animator.SetTrigger("jump");

        if (manager.CheckMapCol(requestedGridPos) || manager.CheckForPlayer(this, requestedGridPos, GridPosToPos(requestedGridPos)))
        {
            if (networkControllerScript != null)
            {
                networkControllerScript.SendFeedback(
                    new List<FeedbackObject>() {
                    new FeedbackObject()
                    {
                        ID = this.ID,
                        Action = "D" +      ((int)color.r*255).ToString("X2") +
                                            ((int)color.g*255).ToString("X2") +
                                            ((int)color.b*255).ToString("X2")
                    }
                    }
                    );
            }
            requestedGridPos[0] = currentGridPos[0];
            requestedGridPos[1] = currentGridPos[1];
            return;

        }
        currentlyMoving = true;
        requestedPos = GridPosToPos(requestedGridPos);


        //  transform.position = new Vector3(transform.position.x, transform.position.y, 0.55f);
        //  rigi.AddForce(new Vector3(dir.x, 1, dir.y) * forceModif, ForceMode.VelocityChange);

    }
    void OnFinishedMoving()
    {
        currentPos = requestedPos;
        currentGridPos = PosToGridPos(currentPos);

        List<FoodController> toEat = manager.foodManager.CheckFood(currentGridPos);
        //change pos 
        foreach (var piece in toEat)
        {
            // foreach (var effect in piece.effects)
            // {

            //     effect.EffectEffect(this);
            //     if (effect != null)
            //     {
            //          effects.Add(effect);
            //     }
            // }
            piece.eatable.OnEat(score, this);



        }

        currentlyMoving = false;
        onCooldown = true;


    }

    public void SetNextMove(int dir)
    {
        if (currentlyMoving == true && jumpTimer / timeOfJump < 0.6f)
        {
            return;
        }
        //convert to dir and next grid pos
        //  requestedGridPos = converted ;
        requestedGridPos[0] = currentGridPos[0];
        requestedGridPos[1] = currentGridPos[1];



        switch (dir)
        {
            case 0:
                requestedGridPos[1] = 1 + currentGridPos[1];
                break;
            case 1:
                requestedGridPos[0] = 1 + currentGridPos[0];
                break;
            case 2:
                requestedGridPos[1] = -1 + currentGridPos[1];
                break;
            case 3:
                requestedGridPos[0] = -1 + currentGridPos[0];
                break;

        }

    }


    Vector2 GridPosToPos(int[] dir)
    {
        // Vector2 vec;
        // vec.x = (-gridLayout.Width / 2) * gridLayout.GetGridCellSize().x + dir[0] * gridLayout.GetGridCellSize().x;
        // vec.y = (-gridLayout.Height / 2) * gridLayout.GetGridCellSize().y + dir[1] * gridLayout.GetGridCellSize().y;
        gridLayout.GetWorldPosFromCellIndexes(dir[0], dir[1], out Vector2 vec);
        return vec; // actual transformation here

    }
    int[] PosToGridPos(Vector2 pos)
    {
        int[] gridPos = new int[2];
        // gridPos[0] = (int)((transform.position.x + (gridLayout.Width / 2) * gridLayout.GetGridCellSize().x) / gridLayout.GetGridCellSize().x);
        // gridPos[1] = (int)((transform.position.y + (gridLayout.Height / 2) * gridLayout.GetGridCellSize().y) / gridLayout.GetGridCellSize().y);
        gridLayout.GetCellIndexesFromWorldPos(pos, out gridPos[0], out gridPos[1]);
        return gridPos; // actual transformation here
    }
    void OnEatFood(IEatable food)
    {
        food.OnEat(score, this);
    }




}
